# R-script used for DESeq2 analysis single-cell RNA-seq data.
# In case of questions contact Jens Johansen - jens.johansen@bric.ku.dk

# R-3.5.1
# Mus musculus, mm10

BASEDIR<-"/NextGenSeqData/project-data/jens/BRIC/aditya_sankar/kdm4a/sc_rnaseq_aug2018"

#########################################################################
## MAKE DESEQ OBJECTS
#########################################################################

library(gplots)
library(RColorBrewer)
library(vsn)
library(DESeq2)

#source custom functions
source(paste(BASEDIR,"scripts/deseq2_pairwise_functions.R",sep="/"))

#load our data
load(paste(BASEDIR,"annotation/gencodeVM18_ann.RData",sep="/")) 
load(paste(BASEDIR,"counts/gencodeVM18_raw_counts.RData",sep="/")) #41300   160 including '00R_Pgap2' gene name

setwd(paste(BASEDIR,"deseq2/pairwise",sep="/"))

##add gene annotation
m<-match(rownames(cd),gene_ann[,"Symbol"])
#any(is.na(m)) #FALSE 
ann<-gene_ann[m,]

##REMOVE samples 8c.ko.4 (extreme low count) and 8c.ko.7 (clusters clearly MII and will weaken stat results)
anno<-anno[!(anno$Sample %in% c("8c.ko.4","8c.ko.7")),]
cd<-cd[,!(anno$Sample %in% c("8c.ko.4","8c.ko.7"))]

##make ALL samples object

dsds_list<-list() #list of DESeqDataSet's
for (cur_stage in levels(anno$Stage)) {

  i<-which(cur_stage==anno$Stage)

  dsds_list[[cur_stage]] <- DESeqDataSetFromMatrix(countData=round(cd)[,i], colData=data.frame(apply(anno[i,],2,as.character)), design=~1)
  colData(dsds_list[[cur_stage]])$Genotype<-relevel(colData(dsds_list[[cur_stage]])$Genotype,ref="wt")
  rowData(dsds_list[[cur_stage]] )<-ann

}

save(list=c("dsds_list","anno","ann"),file="gencodeVM18_raw_pairwise.RData")

##############################################################
### RUN DESEQ2 WITHIN CELL STAGES - WT VS KO
##############################################################

library(ggplot2)
library(gplots)
library(RColorBrewer)
library(vsn)
library(DESeq2)

#source custom functions
source(paste(BASEDIR,"scripts/rnaseq_nov2018_functions.R",sep="/"))

setwd(paste(BASEDIR,"deseq2/pairwise",sep="/"))

load("gencodeVM18_raw_pairwise.RData")

#same filtering thresholds as used by scde
MIN.READS=10
MIN.DETECTED=5

y_list<-list()
res_list<-list()
for (cur_stage in names(y_list)) {

  cat(cur_stage,"\n---------\n")

  lab<-paste(cur_stage,"designGenotype",sep="_")

  x<-dsds_list[[cur_stage]]

  #remove low count. Raw count >10 for all samples in at least one group 
  cat("Applying low count filter\n")
  nonzero_ind<-rowSums(counts(x)) > MIN.READS & rowSums(counts(x) > 0) > MIN.DETECTED
  cat("  Removed",sum(!nonzero_ind),"genes\n") 
  
  #two-factor study
  cat("Fitting DESeq2 model\n")
  design(x)<- ~Genotype
  y<-DESeq(x[nonzero_ind,])
  
  #various QC plots
  deseq2.preprocess.plots(y,lab=lab)
  
  #plot normalized counts of (Kdm4a)
  p<-plotCounts2(y=y,gene="Kdm4a",groupby="Genotype",colorby="Group")
  ggsave(filename=paste("Kdm4a_normcounts_",lab,".pdf",sep=""),p)

  #default
  cat("  Default thresholds\n")
  outlab<-paste(lab,"_lfcThreshold0_alpha0.1",sep="")
  res_list[[cur_stage]]<-data.frame(gene=rownames(y),results(y)[,c("baseMean","log2FoldChange","lfcSE","pvalue","padj")]) #SE=standard error

  y_list[[cur_stage]]<-y
}

#merge pairwise deseq2 results into single dataframe
full_dat<-merge_list_of_dataframes(mylist=res_list,key="gene")
full_dat<-merge(x=ann,y=full_dat,by.x="Symbol",by.y="gene")

colnames(full_dat)<-sub("^XYZ_","",colnames(full_dat))

write.table(x=full_dat,file="deseq2_default_pairwise_full_result_min8c.ko.4_7.txt",quote=FALSE, sep="\t", row.names=FALSE)

save(list=c("y_list","res_list","full_dat"),file="gencodeVM18_deseq2_pairwise_min8c.ko.4_7.RData")

##############################################################
### MAKE GGPLOT FRIENDLY DAT FROM FULL_DAT
##############################################################

library(tidyverse)
library(reshape2)

load("gencodeVM18_deseq2_pairwise_min8c.ko.4_7.RData")
rm(list=c("y_list","res_list"))

#make 4 separate data frames: lfd, lfcse and fdr:
#log2FoldChange,lfcSE,padj,baseMean

type_nms<-c("log2FoldChange","lfcSE","padj","baseMean")

melt_list<-lapply(type_nms,deseq2_res_dat2ggplot_melt,dat=full_dat)
#names(melt_list)<-type_nms

deseq2_long<-Reduce(function(x,y) { merge(x,y,by="key",all=TRUE) }, melt_list)
# Warning message:
# In merge.data.frame(x, y, by = "key", all = TRUE) :
#   column names ‘Gene.x’, ‘Stage.x’, ‘Stage2int.x’, ‘Gene.y’, ‘Stage.y’, ‘Stage2int.y’ are duplicated in the result
#find & rm duplicates columns
v<-sub("\\.[a-z]","",colnames(deseq2_long))
deseq2_long <- deseq2_long[,match(unique(v),v)] %>% select(-key)
#clean colnames
colnames(deseq2_long)<-sub("\\.[a-z]","",colnames(deseq2_long))
#            Gene Stage log2FoldChange Stage2int     lfcSE      padj  baseMean
# 1     00R_Pgap2    2c    -0.54449718         3 0.8463201        NA 0.8049283
# 2     00R_Pgap2    4c    -0.25677128         4 0.3341902 0.6972324 5.1274755
# 3     00R_Pgap2    8c     0.01359064         5 0.3559685 0.9858092 4.0072108

save(list=c("full_dat","deseq2_long"),file="gencodeVM18_deseq2_ggplot_melt.RData")


